# -*- Perl -*-

package SGML::DTDParse::Refentry;

use strict;
require 5.000;
require Carp;
require 'dtdlib/base.pl';

$SGML::DTDParse::Refentry::VERSION = $SGML::DTDParse::VERSION;

@SGML::DTDParse::Refentry::ISA = qw (SGML::DTDParse);

{
    package SGML::DTDParse::DTD_Element;
    # Extension to package defined in DTD.pm

    sub sgml_file {
	my($self, $hash, $name) = @_;
	my($sgml) = "";

	# Usage: ->sgml_file('element', $name) # for element $name
	#     or ->sgml_file('entity', $name)  # for entity $name, etc.
	#     or ->sgml_file()                 # for this

	if (!defined ($hash) || !defined ($name)) {
	    $sgml = &main::Filename(ref $self, $self->name());
	} else {
	    $sgml = &main::Filename($hash, $name);
	}

	$sgml .= ".sgm";
	return $sgml;
    }

    sub print_content_model {
	my($self) = shift;
	local($_) = shift;
	local(*FILE) = shift;

	my(@cm) = $self->format_content_model($_);
	foreach $_ (@cm) {
	    print FILE $self->make_links($_, 1, 1), "\n";
	}
    }

    sub make_links {
	my($self) = shift;
	local($_) = shift;
	my($elements, $entities) = @_;
	my($dtd) = $self->{'DTD'};
	my($out) = "";

	while ($_ =~ /^(.)(.*)$/) {
	    my($first, $rest) = ($1, $2);

#	    print "f: $first\n";
#	    print "r: $rest\n";

	    if (($first eq "%") && $entities) {
		my($match, $name, $url);
		my($dir) = $dtd->entity_dir();

		($match = $rest) =~ /^([A-Z0-9\-\.]+);?/i;
		$name = $1;
		$url  = $1;
		$rest = $';

		$url = $self->sgml_file('entity', $url);
		$url = "../$dir/$url";

		if ($self->{'DTD'}->entity($name)) {
		    $out .= "%<Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}ent.$name\">$name</Link>;";
		} else {
		    $out .= "%$name;";
		}
	    } elsif (($first =~ /[A-Z0-9\-\.]/i) && $elements) {
		my($match, $one, $url, $name);
		my($dir) = $self->{'DTD'}->element_dir();

		if ($rest =~ /^([A-Z0-9\-\.]+)/i) {
		    $match = $1;
		    $name = "$first$1";
		    $url  = "$first$1";
		    $rest = $';

#		    print "Rest: $rest\n";
#		    print "Mtch: $&\n";
#		    print "left: $'\n\n";

		} else {
		    $match = "";
		    $name = $first;
		    $url = $first;
		}

		$url = $self->sgml_file('element', $url);
		$url = "../$dir/$url";

		if ($self->{'DTD'}->element($name)) {
		    $out .= "<Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}elem.$name\">$name</Link>";
		} else {
		    $out .= $name;
		}
	    } else {
		$out .= $first;
	    }

#	    print "F: $first\n";
#	    print "R: $rest\n";
	    
	    $_ = $rest;

#	    print "_: $_\n";
	}

#	print "L: $_\n";
	
	$out .= $_;
	return $out;
    }	

    sub parse_attributes {
	my($self) = shift;
	local($_) = shift;
	my($dtd) = $self->{'DTD'};
	my($pre, $att, $rest, @attr, @exattr);
	my($skip, $skiptext);

	$_ = $dtd->expand_entities($_); # if $dtd->{'EXPAND'};

	@attr = ();
	s/\s+/ /g;
	while ($_ !~ /^\s*$/) {
	    my($mc) = new Text::DelimMatch;
	    s/^\s*(.*?)\s*$/$1/g;

	    if (/^\(/) {
		$mc->delim('\(', '\)');
		($pre, $att, $_) = $mc->match ($_);
	    } elsif (/^\'/) {
		$mc->delim("'");
		($pre, $att, $_) = $mc->match ($_);
	    } elsif (/^\"/) {
		$mc->delim('"');
		($pre, $att, $_) = $mc->match ($_);
	    } else {
		/^(\S+)\s*(.*)$/;
		$pre = "";
		$att = $1;
		$_ = $2;
	    }

	    push (@attr, "$pre$att");
	}
	
	@exattr = ();
	while (@attr) {
	    $att = shift @attr;
	    if ($att =~ /^\%/) {
		push (@exattr, ($att, "", "", ""));
	    } else {
		my($values) = shift @attr;
		my($defval) = shift @attr;
		my($type) = "&nbsp;";

		if (($defval) =~ /^\#fixed/i) {
		    $type = $defval;
		    $defval = shift @attr;
		} elsif ($defval =~ /^\#/i) {
		    $type = $defval;
		    $defval = "&nbsp;";
		}

		if ($values =~ /NOTATION/i) {
		    $type = $values;
		    $values = $defval;
		    $defval = shift @attr;
		}

		$values =~ s/\|/ \| /g; # allow long lists of attribute
		                        # values to break properly in tables

		push (@exattr, ($att, $values, $type, $defval));
	    }
	}

	return @exattr;
    }

    sub format_attributes {
	my($self) = shift;
	local($_) = shift;
	local(*MAN) = shift;
	my($dtd) = $self->{'DTD'};
	my($pretext) = "";
	my($att, $rest, @attr, @exattr, $para);
	my($skip, $skiptext);

	@attr = $self->parse_attributes ($_);

	return if !@attr;

	($para, @attr) = &main::analyze_attributes($dtd, @attr);

	if ($para) {
	    print MAN "<Para>\n";
	    print MAN "$para\n";
	    print MAN "</Para>\n";
	}

	return if !@attr;

	print MAN "<INFORMALTABLE>\n";
	print MAN "<TGROUP COLS=4>\n";
	print MAN "<COLSPEC COLNUM=2 COLNAME=C2>\n";
	print MAN "<COLSPEC COLNUM=4 COLNAME=C4>\n";
	print MAN "<SPANSPEC SPANNAME=C2C4 NAMEST=C2 NAMEEND=C4>\n";
	print MAN "<THEAD>\n";
	print MAN "<ROW>\n";
	print MAN "  <ENTRY><PARA>Name</PARA></ENTRY>\n";
	print MAN "  <ENTRY><PARA>Value(s)</PARA></ENTRY>\n";
	print MAN "  <ENTRY><PARA>Type</PARA></ENTRY>\n";
	print MAN "  <ENTRY><PARA>Default Value</PARA></ENTRY>\n";
	print MAN "</ROW>\n";
	print MAN "</THEAD>\n";
	print MAN "<TBODY>\n";

	while (@attr) {
	    $att = shift @attr;
	    if ($att =~ /^\%/) {
		splice (@attr, 0, 3);
		print MAN "<ROW>\n";
		print MAN "  <ENTRY><PARA>", $self->make_links($att, 0, 1), "</PARA></ENTRY>\n";
		print MAN "  <ENTRY SPANNAME=C2C4>&nbsp;</ENTRY>\n";
		print MAN "</ROW>\n";
	    } else {
		my ($values, $type, $defval) = splice (@attr, 0, 3);

		$att = $self->make_links($att, 0, 1);
		$values = $self->make_links($values, 0, 1);
		$type = $self->make_links($type, 0, 1);
		$defval = $self->make_links($defval, 0, 1);

		print MAN "<ROW>\n";
		print MAN "  <ENTRY><PARA>$att</PARA></ENTRY>\n";
		print MAN "  <ENTRY><PARA>$values</PARA></ENTRY>\n";
		print MAN "  <ENTRY><PARA>$type</PARA></ENTRY>\n";
		print MAN "  <ENTRY><PARA>$defval</PARA></ENTRY>\n";
		print MAN "</ROW>\n";
	    }
	}
	
	print MAN "</TBODY>\n\n";
	print MAN "</TGROUP>\n\n";
	print MAN "</INFORMALTABLE>\n\n";
    }
}

{
    package SGML::DTDParse::ELEMENT;
    # Extension to package defined in DTD.pm

    sub sgml {
	my($self) = shift;
	local(*MAN) = shift;
	my($name) = $self->name();
	my($smin, $emin, $incl, $excl, $cont) = ("", "", "", "", "");
	my($file) = $name;
	my (%min) = ('--' => "Both the start- and end-tags are required for this element.",
		     'OO' => "Both the start- and end-tags are optional for this element, if your SGML declaration allows tag minimization.",
		     'O-' => "The start-tag is optional for this element, if your SGML declaration allows tag minimization.  The end-tag is required.",
		     '-O' => "The start-tag is required for this element.  The end-tag is optional, if your SGML declaration allows minimization."
		    );
	my(@children, @parents, $url);
	my(@ichildren, @iparents, $num_children);
	my($expansion);
	my($refpurpose, $description, $example, $attrdesc, $moresynop);
	my(@pe, $entity);
	local($_, *EXM);

	if ($self->{'MIN'} =~ /^(.) (.)/) {
	    $smin = $1;
	    $emin = $2;
	} else {
	    $smin = "-";
	    $emin = "-";
	}

	$incl = $self->{'INCL'};
	$excl = $self->{'EXCL'};
	$cont = $self->{'CONT'};

	print MAN "<RefEntry ID=\"${main::DTD_ID_MS_PREFIX}elem.$name\">\n";
	print MAN "<!-- Created by DTDParse::Refentry version $SGML::DTDParse::Refentry::VERSION -->\n";
	print MAN "<!-- Consult http://www.berkshire.net/~norm/dtdparse/ -->\n";
	print MAN "<!-- for more information. -->\n\n";

	print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refmeta [\n";
	print MAN "<RefMeta>\n";
	print MAN "<IndexTerm><Primary>$name Element</Primary></IndexTerm>\n";
	print MAN "<RefEntryTitle>$name</RefEntryTitle>\n";
	print MAN "<RefMiscInfo>Element</RefMiscInfo>\n";
	print MAN "</RefMeta>\n";
	print MAN "]]>\n\n";

	print MAN "<RefNameDiv>\n";
	print MAN "<RefName>$name</RefName>\n";

	$refpurpose = &main::RefPurpose($self);

	print MAN "<RefPurpose>\n";
	print MAN "$refpurpose\n";
	print MAN "</RefPurpose>\n";
	print MAN "</RefNameDiv>\n\n";

	print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv [\n";
	print MAN "<RefSynopsisDiv><Title>Content Model</Title>\n";
	print MAN "<Synopsis>\n";
	print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.synopsis [\n";
	$self->print_content_model ($cont, *MAN);
	print MAN "]]>\n";
	print MAN "</Synopsis>\n\n";

	$moresynop = &main::Synopsis($self);
	if ($moresynop ne "") {
	    print MAN $moresynop;
	}

	if ($incl) {
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv.incl [\n";
	    print MAN "<RefSect2><Title>Inclusions</Title>\n";
	    print MAN "<Para>\n";
	    print MAN "$name includes these elements at every level:\n";
	    print MAN "</Para>\n";

	    print MAN "<Synopsis>\n";
	    $self->print_content_model ($incl, *MAN);
	    print MAN "</Synopsis>\n";
	    print MAN "</RefSect2>\n";
	    print MAN "]]>\n\n";
	}

	if ($excl) {
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv.excl [\n";
	    print MAN "<RefSect2><Title>Exclusions</Title>\n";
	    print MAN "<Para>\n";
	    print MAN "$name excludes these elements at every level:\n";
	    print MAN "</Para>\n";

	    print MAN "<Synopsis>\n";
	    $self->print_content_model ($excl, *MAN);
	    print MAN "</Synopsis>\n";
	    print MAN "</RefSect2>\n";
	    print MAN "]]>\n\n";
	}

	if ($self->{'DTD'}->attlist($self->name())) {
	    my($attr) = $self->{'DTD'}->attlist($self->name());

	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv.attr [\n";
	    print MAN "<RefSect2><Title>Attributes</Title>\n";
	    $attr->sgml(*MAN);
	    print MAN "</RefSect2>\n";
	    print MAN "]]>\n\n";
	}

	print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv.tagmin [\n";
	print MAN "<RefSect2><Title>Tag Minimization</Title>\n";
	print MAN "<Para>\n";
	print MAN $min{$smin . $emin}, "\n";
	print MAN "</Para>\n";
	print MAN "</RefSect2>\n";
	print MAN "]]>\n\n";

	@pe = ();
	foreach $entity (values %{$self->{'DTD'}->{'ENTITY'}}) {
	    my($cont) = $entity->{'CONTEX'};
	    next if !$entity->{'PENT'} || $entity->{'FILE'};
	    $cont = " $cont ";
	    $cont =~ s/[\|\&\,]/ /g;
	    push (@pe, $entity) if $cont =~ / $name /i;
	}

	if (!$self->{'DTD'}->{'EXPAND'} && @pe) {
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv.pe [\n";
	    print MAN "<RefSect2><Title>Parameter Entities</Title>\n";
	    print MAN "<Para>\n";
	    print MAN "$name appears in the following parameter entities:\n";
	    print MAN "<SimpleList Type=Inline>\n";

	    while (@pe) { 
		my($entity) = shift @pe;
		my($name) = $entity->name();

		print MAN "<Member>%";
		print MAN "<Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}ent.$name\">$name</Link>";
		print MAN ";</Member>\n";
	    }

	    print MAN "</SimpleList>\n";
	    print MAN "</RefSect2>\n";
	    print MAN "]]>\n\n";
	}

	@parents = sort split(/\s+/, $self->{'PARENTS'});
	@iparents = sort split(/\s+/, $self->{'IPARENTS'});

	@children = split(/\s+/, $self->{'CHILDREN'});
	@ichildren = sort split(/\s+/, $self->{'ICHILDREN'});
	push (@children, @ichildren);

	@children = () if ($#children == 0) && ($children[0] =~ /^empty$/i);

	@children = sort @children;
	$num_children = $#children+1;

	$incl = 0;
	$excl = 0;
	foreach $_ (@parents) {
	    my($elem) = $self->{'DTD'}->element($_);
	    $incl = 1 if ($elem && $elem->{'INCL'});
	    $excl = 1 if ($elem && $elem->{'EXCL'});
	    last if $incl && $excl;
	}

	if (!@children) {
	    # An empty element cannot meaningfully have inclusions or
	    # exclusions...
	    $incl = 0;
	    $excl = 0;
	}

	print MAN "</RefSynopsisDiv>\n";
	print MAN "]]>\n\n";

	$description = &main::Description($self);

	if ($description || @parents || @iparents || @children || $incl) {
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.description [\n";
	    print MAN "<RefSect1><Title>Description</Title>\n";
	    print MAN $description if $description;

	    print MAN "\n\n";
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.description.parents [\n";

	    if (@parents) {
		print MAN "<RefSect2><Title>Parents</Title>\n";

		if ($#parents == 0) {
		    $_ = $parents[0];
		    print MAN "<Para>\n";
		    print MAN "$name occurs in\n"; 
		    print MAN "<Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}elem.$_\">$_</Link>.\n";
		    print MAN "</Para>\n\n";
		} else {
		    print MAN "<Para>\n";
		    print MAN "These elements contain $name:\n";
		    print MAN "<SimpleList Type=Inline>\n";
		    foreach $_ (@parents) {
			print MAN "<Member><Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}elem.$_\">$_</Link></Member>\n";
		    }
		    print MAN "</SimpleList>.\n";
		    print MAN "</Para>\n\n";
		}
	    }

	    if (@iparents) {
		print MAN "<RefSect2><Title>Parents</Title>\n"
		    if !@parents;

		print MAN "<Para>\n";
		print MAN "These elements contain $name at all levels\n";
		print MAN "by inclusion.\n";
		print MAN "</Para>\n\n";
		
		print MAN "<SimpleList Type=Inline>\n";
		foreach $_ (@iparents) {
		    print MAN "<Member><Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}elem.$_\">$_</Link></Member>\n";
		}
		print MAN "</SimpleList>\n";
	    }

	    print MAN "</RefSect2>\n" if @parents || @iparents;
	    print MAN "]]>\n\n";
	
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.description.children [\n";

	    if ($incl || @children) {
		print MAN "<RefSect2><Title>Children</Title>\n";

		if (@children) {
		    if ($#children == 0) {
			$_ = $children[0];
			$_ = "#PCDATA" if /^pcdata$/i;
			print MAN "<Para>\n";
			print MAN "$name contains\n";
			if ($_ eq '#PCDATA'
			    || $_ eq 'RCDATA'
			    || $_ eq 'CDATA') {
			    print MAN "$_.\n";
			} else {
			    print MAN "<Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}elem.$_\">$_</Link>.\n";
			}
			print MAN "</Para>\n\n";
		    } else {
			print MAN "<Para>\n";
			print MAN "$name contains these elements:\n";
			print MAN "<SimpleList Type=Inline>\n";
			foreach $_ (@children) {
			    if ($_ =~ /^pcdata$/i) {
				print MAN "<Member>#PCDATA</Member>\n";
			    } else {
				print MAN "<Member>";
				print MAN "<Link LinkEnd=\"${main::DTD_ID_MS_PREFIX}elem.$_\">$_</Link>";
				print MAN "</Member>\n";
			    }
			}
			print MAN "</SimpleList>.\n";
			print MAN "</Para>\n\n";
		    }
		}

		if ($incl || $excl) {
		    if ($incl) {
			print MAN "<Para>\n";
			print MAN "In some contexts, additional elements ";
			print MAN "may be allowed\n";
			print MAN "by inclusions in parent elements.\n";
		    } 

		    if ($excl) {
			print MAN "<Para>\n" if !$incl;
			if ($incl) {
			    print MAN "Similarly, in ";
			} else {
			    print MAN "In ";
			}

			print MAN "some contexts, some of these elements\n";
			print MAN "may be invalid ";
			print MAN "due to exclusions in parent elements.\n";
		    }

		    print MAN "</Para>\n";
		}
		print MAN "</RefSect2>\n";
	    }
	    print MAN "]]>\n\n";

	    print MAN "\n</RefSect1>\n";
	    print MAN "]]>\n\n";
	}

	if ($self->{'DTD'}->attlist($self->name())) {
	    my($attr) = $self->{'DTD'}->attlist($self->name());
	    my($attrdesc) = &main::AttributeDescription($attr);

	    if ($attrdesc) {
		print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.attrdesc [\n";
		print MAN "<RefSect1><Title>Attributes</Title>\n";
		print MAN "\n$attrdesc\n";
		print MAN "</RefSect1>\n";
		print MAN "]]>\n\n";
	    }
	}

	$example = &main::Example($self);

	if ($example) {
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.examples [\n";
#	    print MAN "<RefSect1><Title>Examples</Title>\n";
	    print MAN "$example\n";
#	    print MAN "</RefSect1>\n";
	    print MAN "]]>\n\n";
	}

	print MAN "</RefEntry>\n";
    }
}

{
    package SGML::DTDParse::ENTITY;
    # Extension to package defined in DTD.pm

    sub sgml {
	my($self) = shift;
	local(*MAN) = shift;
	my($name) = $self->name();
	my($type, $cont, $pent, $file) = ("", "", "", "");
	my($refpurpose);
	my($expansion);
	my($description);
	local($_);

	$type = $self->{'TYPE'};
	$cont = $self->{'CONT'};
	$pent = $self->{'PENT'};
	$file = $self->{'FILE'};

	print MAN "<RefEntry ID=\"${main::DTD_ID_MS_PREFIX}ent.$name\">\n\n";
	print MAN "<!-- Created by DTDParse::Refentry version $SGML::DTDParse::Refentry::VERSION -->\n";
	print MAN "<!-- Consult http://www.berkshire.net/~norm/dtdparse/ -->\n";
	print MAN "<!-- for more information. -->\n\n";

	print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refmeta [\n";
	print MAN "<RefMeta>\n";
	print MAN "<IndexTerm><Primary>$name Parameter Entity</Primary></IndexTerm>\n";
	print MAN "<RefEntryTitle>$name</RefEntryTitle>\n";
	print MAN "<RefMiscInfo>Parameter Entity</RefMiscInfo>\n";
	print MAN "</RefMeta>\n";
	print MAN "]]>\n\n";

	print MAN "<RefNameDiv>\n";
	print MAN "<RefName>$name</RefName>\n";

	$refpurpose = &main::RefPurpose($self);

	print MAN "<RefPurpose>\n";
	print MAN "$refpurpose\n";
	print MAN "</RefPurpose>\n";
	print MAN "</RefNameDiv>\n\n";

	print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.refsynopsisdiv [\n";
	print MAN "<RefSynopsisDiv><Title>Expansion</Title>\n";

	# see if we can figure out that it's an attribute...
	if ($name =~ /attrib$/
	    || $cont =~ /\#IMPLIED/i
	    || $cont =~ /\#REQUIRED/i) {
	    $self->format_attributes ($cont, *MAN);
	} else {
	    print MAN "<Synopsis>\n";
	    $self->print_content_model ($cont, *MAN);
	    print MAN "</Synopsis>\n\n";
	}
	
	print MAN "</RefSynopsisDiv>\n";
	print MAN "]]>\n\n";

	$description = &main::Description($self, 1);

	if ($description) {
	    print MAN "<![ %${main::DTD_ID_MS_PREFIX}ms.description [\n";
	    print MAN "<RefSect1><Title>Description</Title>\n";
	    print MAN $description;
	    print MAN "\n</RefSect1>\n";
	    print MAN "]]>\n\n";
	}

	print MAN "</RefEntry>\n";
    }
}

{
    package SGML::DTDParse::NOTATION;
    # Extension to package defined in DTD.pm

    sub sgml {
	my($self) = @_;
	return;
    }
}

{
    package SGML::DTDParse::ATTLIST;
    # Extension to package defined in DTD.pm

    sub sgml {
	my($self) = shift;
	local(*MAN) = shift;
	my($name) = $self->{'NAME'};
	my($att, $rest, @attr);
	local($_);

	$_ = $self->content();
	return if !defined($_);

	$self->format_attributes($_, *MAN);
    }
}

sub load {
    my($type) = shift;
    my($class) = ref($type) || $type;
    my($self) = load SGML::DTDParse @_;
    return bless $self, $type;
}

sub sgml {
    my($self) = shift;
    my($element) = shift;
    my($name) = shift;
    local(*FILE) = shift;

    if (! exists $self->{$element}->{uc($name)}) {
	warn "There is no $element \"$name\".\n";
	return;
    }

    $self->{$element}->{uc($name)}->sgml(*FILE, @_);
}

sub entity_dir {
    my($self, $expand) = @_;
    my($entities) = $self->{'EXPANDED_ENTITY_DIR'} || "entities";
    my($dtdent)   = $self->{'UNEXPANDED_ENTITY_DIR'} || "dtdent";

    $expand = $self->{'EXPAND'} if !defined($expand);
    
    return $expand ? $entities : $dtdent;
}	

sub element_dir {
    my($self, $expand) = @_;
    my($elements) = $self->{'EXPANDED_ELEMENT_DIR'} || "elements";
    my($dtdelem)  = $self->{'UNEXPANDED_ELEMENT_DIR'} || "dtdelem";

    $expand = $self->{'EXPAND'} if !defined($expand);

    return $expand ? $elements : $dtdelem;
}	

1;

__END__

=head1 NAME

SGML::DTDParse::Refentry - produce a set of DocBook RefEntry pages for a DTD.

=head1 SYNOPSIS

SGML::DTDParse::Refentry is a subclass of SGML::DTDParse

    use SGML::DTDParse::Refentry;

    $dtd = load SGML::DTDParse::Refentry $dtd_database;

    foreach $name ($dtd->elements()) {
       my($element)  = $dtd->element($name);
       my($sgmlfile) = $element->sgml_file();

       print "Writing $name...\n";

       open (SGML, ">$file");
       $element->sgml(*SGML);
       close (SGML);
    }

    # Similarly for entities and notations.

=head1 DESCRIPTION

DTDParse::Refentry prints out a DocBook RefEntry page for each element 
and entity in the DTD.
Attribute lists are automatically presented in the element.  Notations
aren't currently handled, mostly because it isn't clear to me how
they would be usefully presented.

=head1 PUBLIC METHODS

There are no particularly useful public methods of C<SGML::DTDParse::Refentry>

=head1 MEMBER VARIABLES

These variables control some aspects of the translation.  They really should
have access methods, but I haven't bothered to code them up.

=over 4

=item $dtd->{'EXPAND'}

Controls expansion of parameter entities.  If true, all parameter entities
are expanded, but see C<ATTLIST> below.

=item $dtd->{'TITLE'}

The title for the DTD.  Usually something like "DTDName ver#" is sufficient
(e.g., "DocBook 3.0").

=item $dtd->{'SHORTTITLE'}

A short title for the DTD.  Usually just the DTD name.

=item $dtd->{'EXPANDED_ENTITY_DIR'}

=item $dtd->{'UNEXPANDED_ENTITY_DIR'}

=item $dtd->{'EXPANDED_ELEMENT_DIR'}

=item $dtd->{'UNEXPANDED_ELEMENT_DIR'}

=item $dtd->{'EXPANDED_TREE_DIR'}

=item $dtd->{'UNEXPANDED_TREE_DIR'}

When constructing links to other RefEntry pages, C<SGML::DTDParse::Refentry> can
determine the name of the file that will be used for that page, but cannot
know what directories they will be stored in.  It assumes that the
directories specified in these member variables are correct.  The expanded
versions are used if C<$dtd->{'EXPAND'}> is true, otherwise the unexpanded
versions are used.

=back

=head2 Callbacks

These objects use the callbacks described in the documentation for
dtd2man to incorporate additional documentation into the RefEntry pages.

=head1 COPYRIGHT

Copyright (C) 1996 Norman Walsh.  All Rights Reserved.

You are free to use this code in any way you see fit, but give credit
where it's due.

=cut
